"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebhookEventBus = void 0;
const tslib_1 = require("tslib");
const graphql_eventbus_metrics_plugin_1 = require("graphql-eventbus-metrics-plugin");
const graphql_eventbus_core_1 = require("graphql-eventbus-core");
const events_1 = require("events");
class WebhookEventBus {
    constructor(config) {
        this.config = config;
        this.eventEmitter = new events_1.EventEmitter();
        this.init = () => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.bus.init();
        });
        this.publish = (args) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.bus.publish({
                topicName: args.topicName,
                data: args.data,
            });
        });
        this.closeConsumer = () => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.eventEmitter.removeAllListeners();
        });
        this.closePublisher = () => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.eventEmitter.removeAllListeners();
        });
        this.eventEmitter.setMaxListeners(100000);
        this.bus = new graphql_eventbus_core_1.VanillaEventBus({
            plugins: [graphql_eventbus_metrics_plugin_1.metricsPlugin],
            publisher: {
                schema: config.schema,
                publish: (args) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                    this.eventEmitter.emit(`message-${args.topicName}`, args.data);
                }),
                allowInvalidTopic: true,
            },
            subscriber: {
                cb: this.config.cb,
                getSubscription: (topic, cb) => {
                    this.eventEmitter.on(`message-${topic}`, (data) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                        yield cb(data);
                    }));
                },
                queries: this.config.queries,
                schema: this.config.schema,
            },
        });
    }
}
exports.WebhookEventBus = WebhookEventBus;
