import { metricsPlugin } from "graphql-eventbus-metrics-plugin";
import { DocumentNode, GraphQLSchema } from "graphql";
import {
  EventBus,
  Publish,
  VanillaEventBus,
} from "graphql-eventbus-core";

export type WebhookEventBusConfig = {
  schema: GraphQLSchema;
  queries: DocumentNode;
  cb: (p: { key: string; data: {} }) => Promise<unknown>;
};
import { EventEmitter } from "events";
import { DataCb } from "graphql-eventbus-core/build/VanillaEventBus";

export class WebhookEventBus implements EventBus {
  private eventEmitter = new EventEmitter();
  private bus: VanillaEventBus;
  constructor(public config: WebhookEventBusConfig) {
    this.eventEmitter.setMaxListeners(100000);
    this.bus = new VanillaEventBus({
      plugins: [metricsPlugin],
      publisher: {
        schema: config.schema,
        publish: async (args) => {
          this.eventEmitter.emit(
            `message-${args.topicName}`,
            args.data
          );
        },
        allowInvalidTopic: true,
      },
      subscriber: {
        cb: this.config.cb,
        getSubscription: (topic, cb: DataCb) => {
          this.eventEmitter.on(
            `message-${topic}`,
            async (data: {}) => {
              await cb(data);
            }
          );
        },
        queries: this.config.queries,
        schema: this.config.schema,
      },
    });
  }

  init = async () => {
    await this.bus.init();
  };

  publish: Publish = async (args) => {
    await this.bus.publish({
      topicName: args.topicName,
      data: args.data,
    });
  };

  closeConsumer = async () => {
    this.eventEmitter.removeAllListeners();
  };

  closePublisher = async () => {
    this.eventEmitter.removeAllListeners();
  };
}
